@@ -47,6 +47,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <cstdlib>
 #include <cstdio>
 #include <cassert>
+#include <cstring>
+#include <algorithm>
 
 
 /****************************************************************************
@@ -129,42 +131,36 @@ using namespace Async;
 
 
 CppDnsLookupWorker::CppDnsLookupWorker(const string &label)
-  : label(label), worker(0), notifier_rd(-1), notifier_wr(-1),
-    notifier_watch(0), done(false), he_buf(), result(0), buf(0)
+  : label(label), worker_thread(0), notifier_rd(-1), notifier_wr(-1),
+    notifier_watch(0), done(false), result(0)
 {
-  int ret = pthread_mutex_init(&mutex, NULL);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_init: error " << ret << endl;
-  }
 } /* CppDnsLookupWorker::CppDnsLookupWorker */
 
 
 CppDnsLookupWorker::~CppDnsLookupWorker(void)
 {
-  if (worker != 0)
+  if (worker_thread != 0)
   {
-    int ret;
-    
     if (!done)
     {
-      ret = pthread_cancel(worker);
+      int ret = pthread_cancel(worker_thread);
       if (ret != 0)
       {
-	cerr << "pthread_cancel: error " << ret << endl;
+        cerr << "*** WARNING: pthread_cancel: " << strerror(ret) << endl;
       }
     }
-   
-    void *ud;
-    ret = pthread_join(worker, &ud);
+ 
+    int ret = pthread_join(worker_thread, NULL);
     if (ret != 0)
     {
-      cerr << "pthread_join: error " << ret << endl;
+      cerr << "*** WARNING: pthread_join: " << strerror(ret) << endl;
     }
   }
   
-  free(buf);
-  buf = 0;
+  if (result != 0)
+  {
+    freeaddrinfo(result);
+  }
   
   delete notifier_watch;
   if (notifier_rd != -1)
@@ -175,27 +171,15 @@ CppDnsLookupWorker::~CppDnsLookupWorker(void)
   {
     close(notifier_wr);
   }
-  
-  int ret = pthread_mutex_destroy(&mutex);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_destroy: error " << ret << endl;
-  }
 } /* CppDnsLookupWorker::~CppDnsLookupWorker */
 
 
 bool CppDnsLookupWorker::doLookup(void)
 {
-  int ret = pthread_mutex_lock(&mutex);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_lock: error " << ret << endl;
-  }
-  
   int fd[2];
   if (pipe(fd) != 0)
   {
-    perror("pipe");
+    cerr << "*** ERROR: Could not create pipe: " << strerror(errno) << endl;
     return false;
   }
   notifier_rd = fd[0];
@@ -203,26 +187,13 @@ bool CppDnsLookupWorker::doLookup(void)
   notifier_watch = new FdWatch(notifier_rd, FdWatch::FD_WATCH_RD);
   notifier_watch->activity.connect(
       	  mem_fun(*this, &CppDnsLookupWorker::notificationReceived));
-  ret = pthread_create(&worker, NULL, workerFunc, this);
+  int ret = pthread_create(&worker_thread, NULL, workerFunc, this);
   if (ret != 0)
   {
-    cerr << "pthread_create: error " << ret << endl;
+    cerr << "*** ERROR: pthread_create: " << strerror(ret) << endl;
     return false;
   }
-  /*
-  if (pthread_detach(worker) != 0)
-  {
-    perror("pthread_detach");
-    return false;
-  }
-  */
 
-  ret = pthread_mutex_unlock(&mutex);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_unlock: error " << ret << endl;
-  }
-    
   return true;
   
 } /* CppDnsLookupWorker::doLookup */
@@ -266,40 +237,20 @@ void *CppDnsLookupWorker::workerFunc(void *w)
 {
   CppDnsLookupWorker *worker = reinterpret_cast<CppDnsLookupWorker *>(w);
 
-  int ret = pthread_mutex_lock(&worker->mutex);
+  struct addrinfo hints;
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_INET;
+  int ret = getaddrinfo(worker->label.c_str(), NULL, &hints, &worker->result);
   if (ret != 0)
   {
-    cerr << "pthread_mutex_lock: error " << ret << endl;
-  }
-  
-  int bufsize = 512;
-  do
-  {
-    worker->buf = (char *)realloc(worker->buf, bufsize);
-    int h_errnop;
-    ret = gethostbyname_r(worker->label.c_str(), &worker->he_buf, worker->buf,
-      	      	      	  bufsize, &worker->result, &h_errnop);
-    bufsize *= 2;
-  } while (ret == ERANGE);
-  
-  if ((ret != 0) || (worker->result == 0))
-  {
-    free(worker->buf);
-    worker->buf = 0;
-    worker->result = 0;
+    cerr << "*** WARNING: Could not look up host \"" << worker->label
+         << "\": " << gai_strerror(ret) << endl;
   }
   
   ret = write(worker->notifier_wr, "D", 1);
   assert(ret == 1);
   
   worker->done = true;
-  
-  ret = pthread_mutex_unlock(&worker->mutex);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_unlock: error " << ret << endl;
-  }
-    
   return NULL;
   
 } /* CppDnsLookupWorker::workerFunc */
@@ -322,30 +273,30 @@ void CppDnsLookupWorker::notificationReceived(FdWatch *w)
 {
   w->setEnabled(false);
 
-  int ret = pthread_mutex_lock(&mutex);
+  int ret = pthread_join(worker_thread, NULL);
   if (ret != 0)
   {
-    cerr << "pthread_mutex_lock: error " << ret << endl;
+    cerr << "*** WARNING: pthread_join: " << strerror(ret) << endl;
   }
+  worker_thread = 0;
 
   if (result != 0)
   {
-    for (int i=0; result->h_addr_list[i] != NULL; ++i)
+    struct addrinfo *entry;
+    for (entry = result; entry != 0; entry = entry->ai_next)
     {
-      struct in_addr *addr;      
-      addr = reinterpret_cast<struct in_addr *>(result->h_addr_list[i]);
-      the_addresses.push_back(IpAddress(*addr));
+      //printf("ai_family=%d ai_socktype=%d ai_protocol=%d\n",
+      //       entry->ai_family, entry->ai_socktype, entry->ai_protocol);
+      IpAddress ip_addr(
+          reinterpret_cast<struct sockaddr_in*>(entry->ai_addr)->sin_addr);
+      if (find(the_addresses.begin(), the_addresses.end(), ip_addr) ==
+          the_addresses.end())
+      {
+        the_addresses.push_back(ip_addr);
+      }
     }
   }
-  
-  ret = pthread_mutex_unlock(&mutex);
-  if (ret != 0)
-  {
-    cerr << "pthread_mutex_unlock: error " << ret << endl;
-  }
-  
   resultsReady();
-
 } /* CppDnsLookupWorker::notificationReceived */
 
 
