@@ -6,7 +6,7 @@
 
 \verbatim
 Qtel - The Qt EchoLink client
-Copyright (C) 2003-2010 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2015 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <QStringList>
 #include <QComboBox>
 #include <QList>
+#include <QNetworkInterface>
 #undef emit
 
 #include <stdlib.h>
@@ -93,6 +94,7 @@ using namespace std;
 #define CONF_DIRECTORY_SERVER 	      "DirectoryServers"
 #define CONF_LIST_REFRESH_TIME        "ListRefreshTime"
 #define CONF_START_AS_BUSY            "StartAsBusy"
+#define CONF_BIND_ADDRESS             "BindAddress"
 #define CONF_PROXY_ENABLED            "ProxyEnabled"
 #define CONF_PROXY_SERVER             "ProxyServer"
 #define CONF_PROXY_PORT               "ProxyPort"
@@ -211,21 +213,19 @@ Settings *Settings::instance(void)
 
 
 Settings::Settings(void)
-  : dialog(0)
+  : dialog(0),
+    m_list_refresh_time(CONF_LIST_REFRESH_TIME_DEFAULT),
+    m_start_as_busy(CONF_START_AS_BUSY_DEFAULT),
+    m_proxy_enabled(CONF_PROXY_ENABLED_DEFAULT),
+    m_proxy_port(CONF_PROXY_PORT_DEFAULT),
+    m_use_full_duplex(CONF_USE_FULL_DUPLEX_DEFAULT),
+    m_card_sample_rate(CONF_CARD_SAMPLE_RATE_DEFAULT),
+    m_chat_encoding(0),
+    m_vox_enabled(CONF_VOX_ENABLED_DEFAULT),
+    m_vox_threshold(CONF_VOX_THRESHOLD_DEFAULT),
+    m_vox_delay(CONF_VOX_DELAY_DEFAULT)
 {
-  encodings.push_back(Encoding("Big5", SettingsDialog::trUtf8("Chinese")));
-  encodings.push_back(Encoding("Big5-HKSCS", SettingsDialog::trUtf8("Chinese")));
-  encodings.push_back(Encoding("eucJP", SettingsDialog::trUtf8("Japanese")));
-  encodings.push_back(Encoding("eucKR", SettingsDialog::trUtf8("Korean")));
-  encodings.push_back(Encoding("GB2312", SettingsDialog::trUtf8("Chinese")));
-  encodings.push_back(Encoding("GBK", SettingsDialog::trUtf8("Chinese")));
-  encodings.push_back(Encoding("GB18030", SettingsDialog::trUtf8("Chinese")));
-  encodings.push_back(Encoding("JIS7", SettingsDialog::trUtf8("Japanese")));
-  encodings.push_back(Encoding("Shift-JIS", SettingsDialog::trUtf8("Japanese")));
-  encodings.push_back(Encoding("TSCII", SettingsDialog::trUtf8("Tamil")));
-  encodings.push_back(Encoding("utf8", SettingsDialog::trUtf8("Unicode, 8-bit")));
-  encodings.push_back(Encoding("KOI8-R", SettingsDialog::trUtf8("Russian")));
-  encodings.push_back(Encoding("KOI8-U", SettingsDialog::trUtf8("Ukrainian")));
+    // The default encoding should be first in the list
   encodings.push_back(Encoding("ISO8859-1", SettingsDialog::trUtf8("Western")));
   encodings.push_back(Encoding("ISO8859-2", SettingsDialog::trUtf8("Central European")));
   encodings.push_back(Encoding("ISO8859-3", SettingsDialog::trUtf8("Central European")));
@@ -242,6 +242,19 @@ Settings::Settings(void)
   encodings.push_back(Encoding("ISO8859-13"));
   encodings.push_back(Encoding("ISO8859-14"));
   encodings.push_back(Encoding("ISO8859-15", SettingsDialog::trUtf8("Western")));
+  encodings.push_back(Encoding("utf8", SettingsDialog::trUtf8("Unicode, 8-bit")));
+  encodings.push_back(Encoding("Big5", SettingsDialog::trUtf8("Chinese")));
+  encodings.push_back(Encoding("Big5-HKSCS", SettingsDialog::trUtf8("Chinese")));
+  encodings.push_back(Encoding("eucJP", SettingsDialog::trUtf8("Japanese")));
+  encodings.push_back(Encoding("eucKR", SettingsDialog::trUtf8("Korean")));
+  encodings.push_back(Encoding("GB2312", SettingsDialog::trUtf8("Chinese")));
+  encodings.push_back(Encoding("GBK", SettingsDialog::trUtf8("Chinese")));
+  encodings.push_back(Encoding("GB18030", SettingsDialog::trUtf8("Chinese")));
+  encodings.push_back(Encoding("JIS7", SettingsDialog::trUtf8("Japanese")));
+  encodings.push_back(Encoding("Shift-JIS", SettingsDialog::trUtf8("Japanese")));
+  encodings.push_back(Encoding("TSCII", SettingsDialog::trUtf8("Tamil")));
+  encodings.push_back(Encoding("KOI8-R", SettingsDialog::trUtf8("Russian")));
+  encodings.push_back(Encoding("KOI8-U", SettingsDialog::trUtf8("Ukrainian")));
   encodings.push_back(Encoding("IBM850"));
   encodings.push_back(Encoding("IBM866"));
   encodings.push_back(Encoding("CP874"));
@@ -284,6 +297,26 @@ void Settings::showDialog(void)
     }
     settings_dialog.chat_encoding->insertItem(10000, str);
   }
+
+    // Set up bind address combobox
+  foreach (const QNetworkInterface &netif, QNetworkInterface::allInterfaces())
+  {
+    const QString &hname = netif.name();
+    foreach (const QNetworkAddressEntry &netaddr, netif.addressEntries())
+    {
+      const QHostAddress &addr = netaddr.ip();
+      if (addr.protocol() == QAbstractSocket::IPv4Protocol)
+      {
+        QString item_text(addr.toString() + " (" + hname + ")");
+        settings_dialog.bind_address->addItem(item_text, addr.toString());
+      }
+    }
+  }
+  int cur_index = settings_dialog.bind_address->findData(m_bind_address);
+  if (cur_index > 0)
+  {
+    settings_dialog.bind_address->setCurrentIndex(cur_index);
+  }
   
   settings_dialog.my_callsign->setText(m_callsign);
   settings_dialog.my_password->setText(m_password);
@@ -347,6 +380,15 @@ void Settings::showDialog(void)
 	m_directory_servers = settings_dialog.directory_servers->text();
 	m_list_refresh_time = settings_dialog.list_refresh_time->value();
 	m_start_as_busy = settings_dialog.start_as_busy_checkbox->isChecked();
+        int cur_index = settings_dialog.bind_address->currentIndex();
+        if (cur_index > 0)
+        {
+	  m_bind_address = settings_dialog.bind_address->itemData(cur_index).toString();
+        }
+        else
+        {
+          m_bind_address = "";
+        }
 	m_proxy_enabled = settings_dialog.proxy_enabled->isChecked();
 	m_proxy_server = settings_dialog.proxy_server->text();
 	m_proxy_port = settings_dialog.proxy_port->value();
@@ -371,6 +413,7 @@ void Settings::showDialog(void)
 	qsettings.setValue(CONF_DIRECTORY_SERVER, m_directory_servers);
 	qsettings.setValue(CONF_LIST_REFRESH_TIME, m_list_refresh_time);
 	qsettings.setValue(CONF_START_AS_BUSY, m_start_as_busy);
+	qsettings.setValue(CONF_BIND_ADDRESS, m_bind_address);
 	qsettings.setValue(CONF_PROXY_ENABLED, m_proxy_enabled);
 	qsettings.setValue(CONF_PROXY_SERVER, m_proxy_server);
 	qsettings.setValue(CONF_PROXY_PORT, m_proxy_port);
@@ -414,6 +457,7 @@ void Settings::readSettings(void)
       CONF_LIST_REFRESH_TIME_DEFAULT).toInt();
   m_start_as_busy = qsettings.value(CONF_START_AS_BUSY,
       CONF_START_AS_BUSY_DEFAULT).toBool();
+  m_bind_address = qsettings.value(CONF_BIND_ADDRESS).toString();
   m_proxy_enabled = qsettings.value(CONF_PROXY_ENABLED,
       CONF_PROXY_ENABLED_DEFAULT).toBool();
   m_proxy_server = qsettings.value(CONF_PROXY_SERVER,
