@@ -6,7 +6,7 @@
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2003-2014 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2019 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -484,7 +484,7 @@ static const unsigned audio_ch = 0;
 int main(int argc, const char *argv[])
 {
   cout << PROGRAM_NAME " v" DEVCAL_VERSION
-          " Copyright (C) 2003-2015 Tobias Blomberg / SM0SVX\n\n";
+          " Copyright (C) 2003-2019 Tobias Blomberg / SM0SVX\n\n";
   cout << PROGRAM_NAME " comes with ABSOLUTELY NO WARRANTY. "
           "This is free software, and you\n";
   cout << "are welcome to redistribute it in accordance with the "
@@ -640,7 +640,7 @@ int main(int argc, const char *argv[])
     }
     if (wb_mode)
     {
-      ddr->setModulation(Ddr::MOD_WBFM);
+      ddr->setModulation(Modulation::MOD_WBFM);
     }
     DevMeasure *dev_measure = new DevMeasure(ddr->preDemodSampleRate(), 
                                              mod_fqs, ddr->nbFq());
