@@ -8,7 +8,7 @@ This file contains a class that implements a local transmitter.
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2004  Tobias Blomberg / SM0SVX
+Copyright (C) 2004-2019 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
+#include <stdint.h>
 #include <vector>
 
 
@@ -54,6 +55,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
+#include <RefCountingPty.h>
 #include "Tx.h"
 
 
@@ -73,10 +75,13 @@ namespace Async
   class AudioSource;
   class AudioValve;
   class AudioPassthrough;
+  class AudioMixer;
 };
 
 class DtmfEncoder;
 class PttCtrl;
+class HdlcFramer;
+class AfskModulator;
 
 
 /****************************************************************************
@@ -158,12 +163,6 @@ class LocalTx : public Tx
      */
     void setTxCtrlMode(TxCtrlMode mode);
     
-    /**
-     * @brief 	Check if the transmitter is transmitting
-     * @return	Return \em true if transmitting or else \em false
-     */
-    bool isTransmitting(void) const { return is_transmitting; }
-    
     /**
      * @brief 	Enable/disable CTCSS on TX
      * @param 	enable	Set to \em true to enable or \em false to disable CTCSS
@@ -173,11 +172,19 @@ class LocalTx : public Tx
     /**
      * @brief 	Send a string of DTMF digits
      * @param 	digits	The digits to send
+     * @param   duration The tone duration in milliseconds
      */
-    void sendDtmf(const std::string& digits);
+    void sendDtmf(const std::string& digits, unsigned duration=0);
+
+    /**
+     * @brief 	Send a data frame
+     * @param 	msg The frame data
+     */
+    void sendData(const std::vector<uint8_t> &msg);
     
     /**
      * @brief   Set the signal level value that should be transmitted
+     * @param   rx_id  The id of the RX that the signal was received on
      * @param   siglev The signal level to transmit
      *
      * This function does not set the output power of the transmitter but
@@ -186,13 +193,23 @@ class LocalTx : public Tx
      * on a link transmitter to transport signal level measurements to the
      * link receiver.
      */
-    void setTransmittedSignalStrength(float siglev);
+    void setTransmittedSignalStrength(char rx_id, float siglev);
     
+    /**
+     * @brief   Set the transmitter frequency
+     * @param   fq The frequency in Hz
+     */
+    void setFq(unsigned fq);
+
+    /**
+     * @brief   Set the transmitter modulation mode
+     * @param   mod The modulation to set (@see Modulation::Type)
+     */
+    void setModulation(Modulation::Type mod);
+
   private:
-    std::string       	    name;
     Async::Config     	    &cfg;
     Async::AudioIO    	    *audio_io;
-    bool      	      	    is_transmitting;
     Async::Timer      	    *txtot;
     bool      	      	    tx_timeout_occured;
     int       	      	    tx_timeout;
@@ -201,6 +218,10 @@ class LocalTx : public Tx
     DtmfEncoder       	    *dtmf_encoder;
     Async::AudioSelector    *selector;
     Async::AudioValve 	    *dtmf_valve;
+    Async::AudioMixer       *mixer;
+    HdlcFramer              *hdlc_framer;
+    AfskModulator           *fsk_mod;
+    //Async::AudioValve 	    *fsk_valve;
     Async::AudioPassthrough *input_handler;
     PttCtrl   	      	    *ptt_ctrl;
     Async::AudioValve 	    *audio_valve;
@@ -208,12 +229,22 @@ class LocalTx : public Tx
     std::vector<int>        tone_siglev_map;
     Async::Timer            *ptt_hangtimer;
     Ptt                     *ptt;
+    bool                    fsk_trailer_transmitted;
+    char                    last_rx_id;
+    bool                    fsk_first_packet_transmitted;
+    HdlcFramer              *hdlc_framer_ib;
+    AfskModulator           *fsk_mod_ib;
+    RefCountingPty          *ctrl_pty;
+    bool                    audio_dev_keep_open;
     
     void txTimeoutOccured(Async::Timer *t);
     bool setPtt(bool tx, bool with_hangtime=false);
     void transmit(bool do_transmit);
     void allDtmfDigitsSent(void);
     void pttHangtimeExpired(Async::Timer *t);
+    bool preTransmitterStateChange(bool do_transmit);
+    void sendFskSiglev(char rxid, uint8_t siglev);
+    void sendFskDtmf(const std::string &digits, unsigned duration);
 
 };  /* class LocalTx */
 
