@@ -6,7 +6,7 @@
 
 \verbatim
 SvxLink - A Multi Purpose Voice Services System for Ham Radio Use
-Copyright (C) 2003-2008 Tobias Blomberg / SM0SVX
+Copyright (C) 2003-2018 Tobias Blomberg / SM0SVX
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -34,11 +34,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
+#include <stdint.h>
 #include <sigc++/sigc++.h>
 
 #include <string>
 #include <map>
 #include <cassert>
+#include <vector>
 
 
 /****************************************************************************
@@ -47,6 +49,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  ****************************************************************************/
 
+#include <Modulation.h>
 #include <AsyncConfig.h>
 #include <AsyncAudioSource.h>
 
@@ -132,6 +135,12 @@ class Rx : public sigc::trackable, public Async::AudioSource
       MUTE_ALL      //! Mute everything. Also close the audio device.
     } MuteState;
 
+    static const char ID_UNKNOWN = '?';   //! Unknown RX id
+
+    /**
+     * @brief   Translate a mute state to a printable sting
+     * @returns Return a printable version of the given mute state
+     */
     static std::string muteStateToString(MuteState mute_state);
 
     /**
@@ -200,7 +209,7 @@ class Rx : public sigc::trackable, public Async::AudioSource
      * @brief 	Find out RX ID of last receiver with squelch activity
      * @returns Returns the RX ID
      */
-    virtual int sqlRxId(void) const { return 0; }
+    virtual char sqlRxId(void) const { return ID_UNKNOWN; }
     
     /**
      * @brief 	Reset the receiver object to its default settings
@@ -213,6 +222,18 @@ class Rx : public sigc::trackable, public Async::AudioSource
      */
     virtual bool isReady(void) const { return true; }
 
+    /**
+     * @brief   Set the receiver frequency
+     * @param   fq The frequency in Hz
+     */
+    virtual void setFq(unsigned fq) {}
+
+    /**
+     * @brief   Set the receiver modulation mode
+     * @param   mod The modulation to set (@see Modulation::Type)
+     */
+    virtual void setModulation(Modulation::Type mod) {}
+
     /**
      * @brief 	A signal that indicates if the squelch is open or not
      * @param 	is_open \em True if the squelch is open or \em false if not
@@ -245,6 +266,12 @@ class Rx : public sigc::trackable, public Async::AudioSource
      * @param	siglev The new signal level
      */
     sigc::signal<void, float> signalLevelUpdated;
+
+    /**
+     * @brief   A signal that is emitted when digital data have been received
+     * @param   frame The data frame that was received
+     */
+    sigc::signal<void, std::vector<uint8_t>&> dataReceived;
     
     /**
      * @brief	A signal that is emitted to publish a state update event
