@@ -194,6 +194,15 @@ ToneDetector::ToneDetector(float tone_hz, float width_hz, int det_delay_ms)
 } /* ToneDetector::ToneDetector */
 
 
+ToneDetector::~ToneDetector(void)
+{
+  delete det_par;
+  det_par = 0;
+  delete undet_par;
+  undet_par = 0;
+} /* ToneDetector::~ToneDetector */
+
+
 void ToneDetector::reset(void)
 {
   setActivated(false);
@@ -335,7 +344,7 @@ void ToneDetector::setDetectPeakThresh(float thresh)
 {
   if (thresh > 0.0f)
   {
-    det_par->peak_thresh = exp10f(thresh/10.0f);
+    det_par->peak_thresh = powf(10, thresh/10.0f);
   }
   else
   {
@@ -348,7 +357,7 @@ void ToneDetector::setUndetectPeakThresh(float thresh)
 {
   if (thresh > 0.0f)
   {
-    undet_par->peak_thresh = exp10f(thresh/10.0f);
+    undet_par->peak_thresh = powf(10, thresh/10.0f);
   }
   else
   {
