@@ -264,11 +264,21 @@ void WbRxRtlSdr::unregisterDdr(Ddr *ddr)
     // Delete myself if this was the last DDR
   if (ddrs.empty())
   {
+    instances.erase(m_name);
     delete this;
   }
 } /* WbRxRtlSdr::unregisterDdr */
 
 
+void WbRxRtlSdr::updateDdrFq(Ddr *ddr)
+{
+  if (auto_tune_enabled)
+  {
+    findBestCenterFq();
+  }
+} /* WbRxRtlSdr::updateDdrFq */
+
+
 bool WbRxRtlSdr::isReady(void) const
 {
   return (rtl != 0) && rtl->isReady();
